@@ -2,7 +2,7 @@
  * general purpose mouse support for Linux
  *
  * Copyright 1993        ajh@gec-mrc.co.uk (Andrew Haylett)
- * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994-1997   rubini@ipvvis.unipv.it (Alessandro Rubini)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -93,7 +93,7 @@ fd_set selSet, readySet, connSet;
 /*-------------------------------------------------------------------*/
 static inline int open_console(const int mode)
 {
-int fd;
+  int fd;
 
   if ((fd=open(consolename, mode)) < 0)
     oops(consolename);
@@ -103,8 +103,8 @@ int fd;
 /*-------------------------------------------------------------------*/
 static inline int wait_text(int *fdptr)
 {
-int fd;
-int kd_mode;
+  int fd;
+  int kd_mode;
 
   close(*fdptr);
   do
@@ -142,9 +142,9 @@ static inline void selection_copy(int x1, int y1, int x2, int y2, int mode)
  * The approach in "selection" causes a bus error when run under SunOS 4.1
  * due to alignment problems...
  */
-unsigned char buf[6*sizeof(short)];
-unsigned short *arg = (unsigned short *)buf + 1;
-int fd;
+  unsigned char buf[6*sizeof(short)];
+  unsigned short *arg = (unsigned short *)buf + 1;
+  int fd;
 
   buf[sizeof(short)-1] = 2;  /* set selection */
 
@@ -166,8 +166,8 @@ int fd;
 /*-------------------------------------------------------------------*/
 static inline void selection_paste(void)
 {
-char c=3;
-int fd;
+  char c=3;
+  int fd;
 
   fd=open_console(O_WRONLY);
   if (ioctl(fd, TIOCLINUX, &c) < 0)
@@ -178,7 +178,7 @@ int fd;
 /*-------------------------------------------------------------------*/
 static  inline int do_selection(Gpm_Event *event)  /* returns 0, always */
 {
-static int x1=1, y1=1, x2, y2;
+  static int x1=1, y1=1, x2, y2;
 #define UNPOINTER() 0
 
   x2=event->x; y2=event->y;
@@ -234,37 +234,45 @@ static int x1=1, y1=1, x2, y2;
 /* returns 0 if the event has not been processed, and 1 if it has */
 static inline int do_client(Gpm_Cinfo *cinfo, Gpm_Event *event)
 {
-Gpm_Connect info=cinfo->data;
-int fd=cinfo->fd;
-/* value to return if event is not used */
-int res = !(info.defaultMask & event->type); 
-MAGIC_P((static int magic=GPM_MAGIC));
+  Gpm_Connect info=cinfo->data;
+  int fd=cinfo->fd;
+  /* value to return if event is not used */
+  int res = !(info.defaultMask & event->type);
 
-  /* control keys: pass it over */
-  if ((info.minMod & event->modifiers) < info.minMod) return 0;
-  if ((info.maxMod & event->modifiers) < event->modifiers) return 0;
+  /* instead of returning 0, scan the stack of clients */
+  if ((info.minMod & event->modifiers) < info.minMod)
+    goto scan;
+  if ((info.maxMod & event->modifiers) < event->modifiers) 
+    goto scan;
 
-  /* not managed, use default mask */
+  /* if not managed, use default mask */
   if (!(info.eventMask & GPM_BARE_EVENTS(event->type)))
-    return res;
- 
+    if (res)
+      return res;
+    else
+      goto scan;
+
   /* WARNING */ /* This can generate a SIGPIPE... I'd better catch it */
   MAGIC_P((write(fd,&magic, sizeof(int))));
   write(fd,event, sizeof(Gpm_Event));
 
   return info.defaultMask & GPM_HARD ? res : 1; /* HARD forces pass-on */
-}
 
+  scan:
+  if (cinfo->next != 0) 
+    return do_client (cinfo->next, event); /* try the next */
+  return 0; /* no next, not used */
+}
 
 /*-------------------------------------------------------------------*/
 /*-------------------------------------------------------------------*/
 /*-------------------------------------------------------------------*/
 static inline char *getMouseData(int fd, Gpm_Type *type)
 {
-static unsigned char data[32]; /* quite a big margin :) */
-char *edata=data+type->packetlen;
-int howmany=type->howmany;
-int i,j;
+  static unsigned char data[32]; /* quite a big margin :) */
+  char *edata=data+type->packetlen;
+  int howmany=type->howmany;
+  int i,j;
 /*....................................... read and identify one byte */
 
     if (read(fd, data, howmany)!=howmany)
@@ -323,16 +331,16 @@ static int statusC=0; /* clicks */
 static inline int processMouse(int fd, Gpm_Event *event, Gpm_Type *type,
 			       int kd_mode)
 {
-char *data;
-static int fine_dx, fine_dy;
-static int i, m;
-static Gpm_Event nEvent;
-static struct vt_stat stat;
-static struct timeval tv1={0,0}, tv2; /* tv1=0: force first click as single */
-static struct timeval timeout={0,0};
-fd_set fdSet;
-static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
-/* static int buttonlock, buttonlockflag; */
+  char *data;
+  static int fine_dx, fine_dy;
+  static int i, m;
+  static Gpm_Event nEvent;
+  static struct vt_stat stat;
+  static struct timeval tv1={0,0}, tv2; /* tv1==0: first click is single */
+  static struct timeval timeout={0,0};
+  fd_set fdSet;
+  static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
+  /* static int buttonlock, buttonlockflag; */
 
 #define GET_TIME(tv) (gettimeofday(&tv, (struct timezone *)NULL))
 #define DIF_TIME(t1,t2) ((t2.tv_sec -t1.tv_sec) *1000+ \
@@ -537,7 +545,7 @@ static int newB=0, oldB=0, oldT=0; /* old buttons and Type to chain events */
  */
 static int get_data(Gpm_Connect *where, int whence)
 {
-static int i;
+  static int i;
 
 #ifdef GPM_USE_MAGIC
   while ((i=read(whence,&check,sizeof(int)))==4 && check!=GPM_MAGIC)
@@ -563,56 +571,58 @@ static int i;
 
 /*-------------------------------------------------------------------*/
                                  /* returns -1 if closing connection */
-static inline int processRequest(int fd, int vc) 
+static inline int processRequest(Gpm_Cinfo *ci, int vc) 
 {
-int i;
-Gpm_Cinfo *cinfoPtr, *next;
-Gpm_Connect conn;
-static Gpm_Event event;
-static struct vt_stat stat;
+  int i;
+  Gpm_Cinfo *cinfoPtr, *next;
+  Gpm_Connect conn;
+  static Gpm_Event event;
+  static struct vt_stat stat;
 
   PDEBUGG((stderr,"Request on %i (console %i)\n",fd,vc));
   if (vc>MAX_VC) return -1;
 
-  /* Hmm... data on inactive client: drop him */
-  if (vc==-1)                   
-    for (i=0; i<=MAX_VC; i++)          /* loop on consoles */
-      for (cinfoPtr=cinfo[i];
-	   cinfoPtr && cinfoPtr->next;
-	   cinfoPtr=cinfoPtr->next)    /* loop on clients */
-	if (cinfoPtr->next->fd==fd)
-	  {
-	  LOG(("Closing"));
-	  close(fd);
-	  FD_CLR(fd,&connSet);
-	  FD_CLR(fd,&readySet);
-	  next=cinfoPtr->next;
-	  cinfoPtr->next=next->next;
-	  free(next);
-	  return(-1);
-	  }
-
+  /* itz 10-22-96 this shouldn't happen now */
   if (vc==-1) oops("Unknown fd selected");
 
-  i=get_data(&conn,fd);
+  i=get_data(&conn,ci->fd);
 
   if (!i) /* no data */
     {
     LOG(("Closing"));
-    close(fd);
-    FD_CLR(fd,&connSet);
-    FD_CLR(fd,&readySet);
-    cinfoPtr=cinfo[vc];
-    cinfo[vc]=cinfoPtr->next; /* pop the stack */
-    free(cinfoPtr);
-    return(-1);
-    }
+    close(ci->fd);
+    FD_CLR(ci->fd,&connSet);
+    FD_CLR(ci->fd,&readySet);
+    if (cinfo[vc]->fd == ci->fd)
+      {
+      /* it was on top of the stack */
+      cinfoPtr = cinfo[vc];
+      cinfo[vc]=cinfo[vc]->next; /* pop the stack */
+      free(cinfoPtr);
+      return -1;
+      }
+    /* somewhere inside the stack, have to walk it */
+    cinfoPtr = cinfo[vc];
+    while (cinfoPtr && cinfoPtr->next) 
+      {
+      if (cinfoPtr->next->fd == ci->fd) 
+	{
+	next = cinfoPtr->next;
+	cinfoPtr->next = next->next;
+	free (next);
+	break;
+	}
+      cinfoPtr = cinfoPtr->next;
+      }
+    return -1;
+    } /* not data */
+ 
   if (i==-1)
     return -1; /* too few bytes */
 
   if (conn.pid!=0)
     {
-    cinfo[vc]->data = conn;
+    ci->data = conn;
     return 0;
     }
 
@@ -636,11 +646,11 @@ static struct vt_stat stat;
 
     case GPM_REQ_BUTTONS:
         event.type= (opt_three==1 ? 3 : 2); /* buttons */
-        write(fd,&event,sizeof(Gpm_Event));
+        write(ci->fd,&event,sizeof(Gpm_Event));
 	break;
 
     case GPM_REQ_CONFIG:
-	xfer_options(1,fd);
+	xfer_options(1,ci->fd);
 	break;
     }
 
@@ -650,11 +660,16 @@ static struct vt_stat stat;
 /*-------------------------------------------------------------------*/
 static inline int processConn(int fd) /* returns newfd or -1 */
 {
-Gpm_Cinfo *info;
-Gpm_Connect *request;
-Gpm_Cinfo *next;
-int vc, newfd, len;
-struct sockaddr_un addr; /* reuse this each time */
+  Gpm_Cinfo *info;
+  Gpm_Connect *request;
+  Gpm_Cinfo *next;
+  int vc, newfd;
+#if !defined(__GLIBC__)
+  int len;
+#else /* __GLIBC__ */
+  size_t len;
+#endif /* __GLIBC__ */
+  struct sockaddr_un addr; /* reuse this each time */
 
 /*....................................... Accept */
 
@@ -710,8 +725,8 @@ struct sockaddr_un addr; /* reuse this each time */
 /*-------------------------------------------------------------------*/
 void get_console_size(Gpm_Event *ePtr)
 {
-int i;
-struct mouse_features *which_mouse; /* local */
+  int i;
+  struct mouse_features *which_mouse; /* local */
 
   i=open_console(O_RDONLY);
   ioctl(i, TIOCGWINSZ, &win);
@@ -740,13 +755,13 @@ struct mouse_features *which_mouse; /* local */
 /*-------------------------------------------------------------------*/
 int main(int argc, char **argv)
 {
-int mousefd, ctlfd, newfd;
-struct sockaddr_un ctladdr;
-int i, len, kd_mode;
-struct   timeval timeout;
-int maxfd=-1;
-int pending;
-Gpm_Event event;
+  int mousefd, ctlfd, newfd;
+  struct sockaddr_un ctladdr;
+  int i, len, kd_mode;
+  struct   timeval timeout;
+  int maxfd=-1;
+  int pending;
+  Gpm_Event event;
 
   prgname=argv[0];
   setuid(0); /* just in case... */
@@ -846,8 +861,12 @@ Gpm_Event event;
 
       /* and notify clients */
       for(i=0; i<MAX_VC+1; i++)
-	if (cinfo[i])
-	  kill(cinfo[i]->data.pid,SIGWINCH);
+	{
+	Gpm_Cinfo *ci;
+
+        for (ci = cinfo[i]; ci; ci = ci->next)
+	  kill(ci->data.pid,SIGWINCH);
+	}
       }
 
     if (pending<0)
@@ -923,25 +942,35 @@ Gpm_Event event;
 
 /*....................................... got request */
 
+    /* itz 10-22-96 check _all_ clients, not just those on top! */
     for (i=0; pending && (i<=MAX_VC); i++)
-      if (cinfo[i] && FD_ISSET(cinfo[i]->fd,&selSet))
+      {
+      Gpm_Cinfo* ci;
+
+      for (ci = cinfo[i]; pending && ci; ci = ci->next) 
 	{
-	int fd=cinfo[i]->fd;
-	PDEBUGG((stderr,"Request\n"));
-        FD_CLR(fd,&selSet); pending--;
-	if ((processRequest(fd,i)==-1) && maxfd==fd)
-	  maxfd--; /* WARN */
-        }
+        if (FD_ISSET(ci->fd,&selSet)) 
+	  {
+          FD_CLR(ci->fd,&selSet); pending--;
+          if ((processRequest(ci,i)==-1) && maxfd==ci->fd)
+            maxfd--;
+	  }
+	}
+      }
+
 
 /*....................................... look for a spare fd */
 
+    /* itz 10-22-96 this shouldn't happen now! */
     for (i=0; pending && i<=maxfd; i++)
+      {
       if (FD_ISSET(i,&selSet))
 	{
-	FD_CLR(i,&selSet);
+        FD_CLR(i,&selSet);
         pending--;
-        processRequest(i,-1); /* WARN *//*maxfd*/
-        }
+        LOG(("Data on strange file descriptor %d", i));
+	}
+      }
 	     
 /*....................................... all done. */
     
