@@ -34,7 +34,12 @@
 #include <sys/types.h>     /* socket() */
 #include <sys/socket.h>    /* socket() */
 #include <sys/un.h>        /* struct sockaddr_un */
-#include <linux/fs.h>      /* MAJOR */
+#include <asm/types.h>     /* __u32  */
+#if defined(__GLIBC__)
+#  include <linux/kdev_t.h>      /* MAJOR */
+#else /* __GLIBC__ */
+#  include <linux/fs.h>      /* MAJOR */
+#endif /* __GLIBC__ */
 
 #ifdef	SIGTSTP		/* true if BSD system */
 #include <sys/file.h>
@@ -235,15 +240,13 @@ static int usage(char *whofailed)
  "    -a accel         sets the acceleration (default %d)\n"
  "    -b baud-rate     sets the baud rate (default %d)\n"
  "    -B sequence      allows changing the buttons (default '%s')\n"
- "    -d delta         sets the delta value (default %d)\n"
- "    -D               dirty operation (debug only)\n"
+ "    -d delta         sets the delta value (default %d) (must be 2 or more)\n"
  "    -g tap-button    sets the button (1-3) that is emulated by tapping on\n"
  "                     a glidepoint mouse, none by default. (mman/ps2 only)\n"
  "    -i interval      max time interval for multiple clicks (default %i)\n"
  "    -k               kill a running gpm, to start X with a busmouse\n"
  "    -K               run in kernel mode, using the kmouse module\n"
  "    -l charset       loads the inword() LUT (default '%s')\n"
- "    -L charset       load LUT and exit\n"
  "    -m mouse-device  sets mouse device\n"
  "    -M               enable multiple mouse. Following options refer to\n"
  "                     the second device. Forces \"-R\"\n"
@@ -323,7 +326,7 @@ return 0; /* never */
 /*===================================================================*/
 int cmdline(int argc, char **argv)
 {
-char options[]="a:b:B:d:Dg:hi:Kkl:L:m:Mo:pqr:Rs:S:t:Tv23";
+char options[]="a:b:B:d:Dg:hi:Kkl:m:Mo:pqr:Rs:S:t:Tv23";
 int i, opt, fd, kfd;
 struct kmouse_options koptions;
 FILE *f;
@@ -410,7 +413,7 @@ static struct {char *in; char *out;} seq[] = {
     {
     which_mouse=mouse_table+i; /* used to access options */
 
-    if (opt_accel<2) exit(usage("acceleration"));
+    if (opt_accel<1) exit(usage("acceleration"));
     if (opt_delta<2) exit(usage("delta"));
     if (strlen(opt_sequence)!=3 || atoi(opt_sequence)<100)
       exit(usage("sequence"));
