@@ -1,7 +1,7 @@
 /*
  * libcurses.c - client library - curses level (gpm-Linux)
  *
- * Copyright 1994   rubini@ipvvis.unipv.it (Alessandro Rubini)
+ * Copyright 1994,1995   rubini@ipvvis.unipv.it (Alessandro Rubini)
  * 
  * xterm management is mostly by Miguel de Icaza
  *
@@ -79,7 +79,7 @@ static Gpm_Event ev;
 #define DELAY_MS 100
     static struct timeval to={0,DELAY_MS*1000};
     static fd_set selSet;
-    static int prevchar=EOF, clicks=0;
+    static int prevchar=EOF;
     extern gpm_convert_event(char *data, Gpm_Event *event);
     int c; char mdata[4];
 
@@ -127,4 +127,5 @@ static Gpm_Event ev;
 	}
       } /* while(1) */
     }
+  return 0;
 }
