@@ -38,6 +38,7 @@
 #include <sys/ioctl.h>
 #endif
 
+#include <signal.h> 
 #include <linux/vt.h>      /* VT_GETSTATE */
 #include <sys/kd.h>        /* KDGETMODE */
 #include <termios.h>       /* winsize */
@@ -96,6 +97,48 @@ static int magic=GPM_MAGIC;
   return 0;
 }
 
+#if (defined(SIGTSTP)) 
+/* itz: support for SIGTSTP */
+
+/* Old SIGTSTP handler. */
+
+static __sighandler_t gpm_saved_suspend_hook;
+
+static void gpm_suspend_hook ()
+{
+	Gpm_Connect gpm_connect;
+	sigset_t old_sigset;
+	sigset_t new_sigset;
+	int success;
+
+	sigemptyset (&new_sigset);
+	sigaddset (&new_sigset, SIGTSTP);
+	sigprocmask (SIG_BLOCK, &new_sigset, &old_sigset);
+
+	/* Open a completely transparent gpm connection */
+	gpm_connect.eventMask = 0;
+	gpm_connect.defaultMask = ~0;
+	gpm_connect.minMod = ~0;
+	gpm_connect.maxMod = 0;
+	/* cannot do this under xterm, tough */
+	success = (Gpm_Open (&gpm_connect, 0) >= 0);
+
+	/* take the default action, whatever it is (probably a stop :) */
+	sigprocmask (SIG_SETMASK, &old_sigset, 0);
+	signal (SIGTSTP, gpm_saved_suspend_hook);
+	kill (getpid (), SIGTSTP);
+	/* in bardo here */
+
+	/* Reincarnation. Prepare for another death early. */
+	signal (SIGTSTP, gpm_suspend_hook);
+	/* Pop the gpm stack by closing the useless connection */
+	/* but do it only when we know we opened one.. */
+	if (success) {
+		Gpm_Close ();
+	} /*if*/
+}
+#endif /* SIGTSTP */
+
 /*-------------------------------------------------------------------*/
 int Gpm_Open(Gpm_Connect *conn, int flag)
 {
@@ -159,7 +202,7 @@ Gpm_Stst *new;
       conn->vc=atoi(tty+8);
       }
     else /* a default handler -- use console */
-      sprintf(tty,"/dev/console");
+      sprintf(tty,"/dev/tty0");
 
     if (gpm_consolefd==-1)
       if ((gpm_consolefd=open(tty,O_WRONLY))<0)
@@ -218,7 +261,15 @@ Gpm_Stst *new;
 /*....................................... Put your data */
 
   if (putdata(gpm_fd,conn)!=-1)
+    {
+#if (defined(SIGTSTP))
+    if (gpm_flag == 1)
+      /* Install suspend hook */
+      if ((gpm_saved_suspend_hook = signal (SIGTSTP, SIG_IGN)) != SIG_IGN)
+	signal (SIGTSTP, gpm_suspend_hook);
+#endif
     return gpm_fd;
+    }
 
 /*....................................... Error: free all memory */
 err:
