@@ -86,7 +86,7 @@ static int M_ms(Gpm_Event *state,  unsigned char *data)
 
   /* Allow motion *and* button change (Michael Plass) */
 
-  if ((state->dx==0) && (state->dx==0) 
+  if ((state->dx==0) && (state->dy==0) 
       && (state->buttons == (prev&~GPM_B_MIDDLE)))
     state->buttons = prev^GPM_B_MIDDLE;  /* no move or change: toggle middle */
   else
@@ -99,6 +99,19 @@ static int M_ms(Gpm_Event *state,  unsigned char *data)
   return 0;
 }
 
+/* m$ 'Intellimouse' (steveb 20/7/97) */
+static int M_ms3(Gpm_Event *state,  unsigned char *data)
+{
+  state->buttons= ((data[0] & 0x20) >> 3)	/* left */
+		| ((data[3] & 0x10) >> 3)	/* middle */
+		| ((data[0] & 0x10) >> 4);	/* right */
+  state->dx=      (char)(((data[0] & 0x03) << 6) | (data[1] & 0x3F));
+  state->dy=      (char)(((data[0] & 0x0C) << 4) | (data[2] & 0x3F));
+  /* wheel (dz??) is (data[3] & 0x0f) */
+
+  return 0;
+}
+
 static int M_bare(Gpm_Event *state,  unsigned char *data)
 {
   /* a bare ms protocol */
@@ -124,6 +137,14 @@ static int M_msc(Gpm_Event *state,  unsigned char *data)
   return 0;
 }
 
+static int M_logimsc(Gpm_Event *state,  unsigned char *data)	/* Equal to msc	*/
+{
+  state->buttons= (~data[0]) & 0x07;
+  state->dx=      (char)(data[1]) + (char)(data[3]);
+  state->dy=     -((char)(data[2]) + (char)(data[4]));
+  return 0;
+}
+
 static int M_mm(Gpm_Event *state,  unsigned char *data)
 {
   state->buttons= data[0] & 0x07;
@@ -164,8 +185,21 @@ static int M_ps2(Gpm_Event *state,  unsigned char *data)
       state->buttons = tap_active = 0;
     else state->buttons = tap_active;
 
-  state->dx=   (data[0] & 0x10) ? data[1]-256 : data[1];
-  state->dy= -((data[0] & 0x20) ? data[2]-256 : data[2]);
+ /* Some PS/2 mice send reports with negative bit set in data[0]
+  * and zero for movement.  I think this is a bug in the mouse, but
+  * working around it only causes artifacts when the actual report is -256;
+  * they'll be treated as zero. This should be rare if the mouse sampling
+  * rate is set to a reasonable value; the default of 100 Hz is plenty.
+  * (Stephen Tell)
+  */
+  if(data[1] != 0)
+    state->dx=   (data[0] & 0x10) ? data[1]-256 : data[1];
+  else
+    state->dx = 0;
+  if(data[2] != 0)
+    state->dy= -((data[0] & 0x20) ? data[2]-256 : data[2]);
+  else
+    state->dy = 0;
   return 0;
 }
 
@@ -240,7 +274,7 @@ static int M_wacom(Gpm_Event *state, unsigned char *data)
   x = (data[1]<<7) | (data[2]&0x7f);
   y = (data[4]<<7) | (data[5]&0x7f);
   z = data[6]-64 + 128*(data[6]<64);
-  if (ox==-1 || abs(x-ox)>128 || abs(y-oy)>128) { ox=x; oy=y;} /* enter */
+  if (ox==-1 || abs(x-ox)>18 || abs(y-oy)>18) { ox=x; oy=y;} /* enter */
 
   state->buttons = (GPM_B_MIDDLE * !!(data[3]&0x10)); /* the b. means middle */
   if (!state->buttons)
@@ -392,6 +426,9 @@ fd_set set; struct timeval timeout={0,0}; /* used when not debugging */
 	break;
 	}
 
+  if (type->fun==M_logimsc)
+    write(fd, "QU", 2 );
+
   if (type->fun==M_ms && i==2 && c==0x33) /* Aha.. a mouseman... */
 	{
 	LOG(("MouseMan detected"));
@@ -454,6 +491,33 @@ static Gpm_Type *I_wacom(int fd, unsigned short flags, struct Gpm_Type *type)
   return type;
 }
 
+static Gpm_Type *I_pnp(int fd, unsigned short flags, struct Gpm_Type *type)
+{  
+  struct termios tty;
+  char *c;
+
+  /*
+   * Just put the device to 1200 baud. Thanks to Francois Chastrette
+   * for his great help and debugging with his own pnp device.
+   */
+  tcgetattr(fd, &tty);
+    
+  tty.c_iflag = IGNBRK | IGNPAR;
+  tty.c_oflag = 0;
+  tty.c_lflag = 0;
+  tty.c_line = 0;
+  tty.c_cc[VTIME] = 0;
+  tty.c_cc[VMIN] = 1;
+  tty.c_cflag = flags | B1200;
+  tcsetattr(fd, TCSAFLUSH, &tty); /* set parameters */
+
+  /*
+   * Don't read the silly initialization string. I don't want to see
+   * the vendor name: it is only propaganda, with no information.
+   */
+  return type;
+}
+
 
 /*========================================================================*/
 /* Finally, the table */
@@ -483,13 +547,13 @@ Gpm_Type mice[]={
   {"mman", "The \"MouseMan\" and similar devices (3/4 bytes per packet).",
            "Mouseman", M_mman, I_serial, PROTO_MMAN, CS7 | STD_FLG, /* first */
                                 {0x40, 0x40, 0x40, 0x00}, 3, 1, 1, 0},
-  {"ms",   "The original micro$oft protocol, with a middle-button extension.",
+  {"ms",   "The original ms protocol, with a middle-button extension.",
            "", M_ms, I_serial, PROTO_MS, CS7 | STD_FLG,
                                 {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0},
-  {"bare", "The ms protocol, unextended. Necessary for some 2-buttons mice.",
+  {"bare", "Unadorned ms protocol. Needed with some 2-buttons mice.",
            "Microsoft", M_bare, I_serial, PROTO_BARE, CS7 | STD_FLG,
                                 {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0},
-  {"msc",  "Mouse-Systems-Compatible. Used in most 3-button mice (5-bytes).",
+  {"msc",  "Mouse-Systems-Compatible (5bytes). Most 3-button mice.",
            "MouseSystems", M_msc, I_serial, PROTO_MSC, CS8 | CSTOPB | STD_FLG,
                                 {0xf8, 0x80, 0x00, 0x00}, 5, 1, 0, 0},
   {"sun",  "'msc' protocol, but only 3 bytes per packet.",
@@ -513,7 +577,15 @@ Gpm_Type mice[]={
   {"wacom",  "Wacom tablet",
            "", M_wacom, I_wacom, PROTO_WACOM, STD_FLG,
                                 {0x80, 0x80, 0x80, 0x00}, 7, 7, 0, 0},
-
+  {"logim",  "Turn logitech into Mouse-Systems-Compatible.",
+           "", M_logimsc, I_serial, PROTO_MSC, CS8 | CSTOPB | STD_FLG,
+                                {0xf8, 0x80, 0x00, 0x00}, 5, 1, 0, 0},
+  {"pnp",  "Plug and pray. New mice may not run with '-t ms'.",
+           "", M_bare, I_pnp, PROTO_BARE, CS7 | STD_FLG,
+                                {0x40, 0x40, 0x40, 0x00}, 3, 1, 0, 0},
+  {"ms3",   "Micro$oft Intellimouse - 3 buttons (and a wheel: NYI).",
+           "", M_ms3, I_pnp, PROTO_MS3, CS7 | STD_FLG,
+                                {0xc0, 0x40, 0xc0, 0x00}, 4, 1, 0, 0},
   {"",     "",
            "", NULL, NULL, PROTO_UNKNOWN, 0,
                                 {0x00, 0x00, 0x00, 0x00}, 0, 0, 0, 0}
@@ -536,9 +608,3 @@ Gpm_Type *type;
 
 return 1; /* to exit() */
 }
-
-
-
-
-
-
