@@ -52,11 +52,11 @@ by 't-mouse.el")
       (set-buffer buffer)
       (goto-char 0)
       (or
+       ;; Many versions of "ps", all different....
        (re-search-forward "p \\([0-9a-f]\\)" nil t)
        (re-search-forward "v0\\([0-9a-f]\\)" nil t)
-       ;; Hmmmm... ps-0.99 changed it again. No key letter any more
-       ;; I must look for the second field...
-       (re-search-forward "[0-9]+ +\\([0-9]+\\)" nil t))
+       (re-search-forward "[0-9]+ +\\([0-9]+\\)" nil t)
+       (re-search-forward "[\\t ]*[0-9]+[\\t ]+\\([0-9a-f]\\)" nil t))
       (and
        (match-beginning 1)
        (setq tty (buffer-substring (match-beginning 1) (match-end 1)))))
